@@ -1,5 +1,5 @@
 /*
-  maeFat Rev 1.1
+  maeFat Rev 1.2
   Ken Young (orrery.moko@gmail.com)
   First Version Feb. 5, 2011
 
@@ -66,8 +66,48 @@
 #define kg_per_lb (0.45359237)
 #define m_per_in (2.54e-2)
 
+#define RENDER_CENTER_X (80)
+#define RENDER_CENTER_Y (400)
+#define INT_TOKEN     0
+#define DOUBLE_TOKEN  1
+#define FLOAT_TOKEN   2
+#define STRING_TOKEN  3
+#define BORDER_FACTOR_X (0.005)
+#define BORDER_FACTOR_Y (0.015)
+#define LEFT_BORDER   (33)
+#define RIGHT_BORDER  (40)
+#define TOP_BORDER    (21)
+#define BOTTOM_BORDER (48)
+
+unsigned char normalYear[12] = {0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5};
+unsigned char leapYear[2] = {6, 2};
+char *homeDir, *userDir, *fileName, *settingsFileName;
+char *backupDir = "/media/mmc1/.maeFat";
+char *backupFile = "/media/mmc1/.maeFat/data";
+char *defaultComment = "Type Comment Here";
+char *monthName[12] = {"January",   "February", "March",    "April",
+		       "May",       "June",     "July",     "August",
+		       "September", "October",  "November", "December"};
+char *dayName[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
+char *maeFatVersion = "1.2";
+char scratchString[200];
+
 int debugMessagesOn = FALSE;
 #define dprintf if (debugMessagesOn) printf
+int nDataPoints = 0;
+int goodColor = OR_GREEN;
+int badColor  = OR_RED;
+int displayHeight, displayWidth, cancelled, deleted;
+int minSelectorWeight = 50;
+int maxSelectorWeight = 300;
+int mainBoxInTrendStackable = FALSE;
+
+double monthLengths[12] = {31.0, 28.0, 31.0, 30.0, 31.0, 30.0, 31.0, 31.0, 30.0, 31.0, 30.0, 31.0};
+double maxJD = -1.0;
+double minJD = 1.0e30;
+double maxWeight = -1.0;
+double minWeight = 1.0e30;
+double xScale, yScale, plotMaxX, plotMinX, plotMaxY, plotMinY;
 
 typedef struct logEntry {
   double time;           /* The time at which the log entry was made */
@@ -79,12 +119,18 @@ typedef struct logEntry {
 
 logEntry *logRoot = NULL;
 logEntry *lastEntry = NULL;
-int nDataPoints = 0;
+logEntry *editPtr;
 
-char *homeDir, *userDir, *fileName, *settingsFileName;
-char *backupDir = "/media/mmc1/.maeFat";
-char *backupFile = "/media/mmc1/.maeFat/data";
+typedef struct logCell {
+  GdkPoint        box[4];
+  logEntry       *ptr;
+  struct logCell *next;
+} logCell;
+logCell *logCellRoot = NULL;
+logCell *lastLogCell = NULL;
+int logDisplayed = FALSE;
 
+/* Values which are stored in the settings file */
 double myHeight          =  72.0;
 double myTarget          = 150.0;
 int weightkg             = FALSE;
@@ -95,30 +141,11 @@ int hackerDietMode       = FALSE;
 int showComments         = FALSE;
 int showTarget           = FALSE;
 int plotInterval         =   365;
-
-int goodColor = OR_GREEN;
-int badColor  = OR_RED;
-
-char *defaultComment = "Type Comment Here";
-
-char *monthName[12] = {"January",   "February", "March",    "April",
-		       "May",       "June",     "July",     "August",
-		       "September", "October",  "November", "December"};
-char *dayName[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
-double monthLengths[12] = {31.0, 28.0, 31.0, 30.0, 31.0, 30.0, 31.0, 31.0, 30.0, 31.0, 30.0, 31.0};
-char *maeFatVersion = "1.1";
-char scratchString[200];
-
-int displayHeight, displayWidth;
-double maxJD = -1.0;
-double minJD = 1.0e30;
-double maxWeight = -1.0;
-double minWeight = 1.0e30;
-double xScale, yScale, plotMaxX, plotMinX, plotMaxY, plotMinY;
+/* End of settings file variables */
 
 GdkGC *gC[N_COLORS];
 GtkWidget *window, *mainBox, *drawingArea, *weightButton, *dataEntryCancel, *dataEditDelete,
-  *dataEntryButton, *chartButton, *logButton, *trendButton, *utilitiesButton, *settingsButton,
+  *dataEntryButton, *logButton, *trendButton, *settingsButton,
   *helpButton, *aboutButton, *commentText, *logSelectorButton;
 GdkPixmap *pixmap = NULL;
 GdkPixmap *cairoPixmap = NULL;
@@ -146,12 +173,21 @@ cairo_t *cairoContext = NULL;
 
 int cairoInitialized = FALSE;
 
+GtkWidget *selector, *dataEntryStackable, *settingsStackable, *logStackable,
+  *dateButton, *timeButton, *trendStackable, *dateEditButton, *timeEditButton,
+  *dataEditStackable, *heightSpinLabel, *heightSpin, *weightUnitLabel, *heightUnitLabel,
+  *dateFormatLabel, *poundsButton, *kgButton, *inButton, *cmButton, *dayButton,
+  *monthButton, *nonjudgementalButton, *hackerDietButton, *showCommentsButton,
+  *plotFortnightButton, *plotMonthButton, *plotQuarterButton, *plot6MonthButton,
+  *plotYearButton, *plotHistoryButton, *targetSpinLabel, *targetSpin, *showTargetButton;
+
+HildonAppMenu *hildonMenu;
+
 /*   E N D   O F   V A R I A B L E   D E C L A R A T I O N S   */
 
-#define RENDER_CENTER_X (80)
-#define RENDER_CENTER_Y (400)
-
 /*
+  C R E A T E  P A N G O  C A I R O  W O R K S P A C E
+
   Create the structures needed to be able to yse Pango and Cairo to
 render antialiased fonts
 */
@@ -164,9 +200,11 @@ void createPangoCairoWorkspace(void)
   }
   cairo_translate(cairoContext, RENDER_CENTER_X, RENDER_CENTER_Y);
   cairoInitialized = TRUE;
-}
+} /* End of  C R E A T E  P A N G O  C A I R O  W O R K S P A C E */
 
 /*
+  R E N D E R  P A N G O  T E X T
+
   The following function renders a text string using Pango and Cairo, on the
   cairo screen, and copies it to a gtk drawable.   The height and width
   of the area occupied by the text is returned in passed variables.
@@ -302,9 +340,11 @@ void renderPangoText(char *theText, unsigned short color, int font,
       gdk_draw_drawable(dest, gC[OR_BLUE], cairoPixmap, RENDER_CENTER_X+xC, RENDER_CENTER_Y+yC,
 			x, y-((*height)>>1), wC, hC);
   }
-}
+} /* End of  R E N D E R  P A N G O  T E X T */
 
 /*
+  C A L C U L A T E  J U L I A N  D A T E
+
   calculateJulianDate converts the Gregorian Calendar date (dd/mm/yyy)
   to the Julian Date, which it returns.
  */
@@ -316,9 +356,11 @@ int calculateJulianDate(int dd, int mm, int yyyy)
   y = yyyy + 4800 - a;
   m = mm + 12*a - 3;
   return(dd + (153*m + 2)/5 + 365*y +y/4 - y/100 + y/400 -32045);
-}
+} /* End of  C A L C U L A T E  J U L I A N  D A T E */
 
 /*
+  E N Q U E U E  E N T R Y
+
   enqueueEntry adds a new log entry to the end of the linked
   list of log entries.
 */
@@ -341,13 +383,22 @@ void enqueueEntry(double jD, float weight, char *comment)
   else
     lastEntry->next = newEntry;
   lastEntry = newEntry;
-}
+} /* End of  E N Q U E U E  E N T R Y */
 
+/*
+  C A L C U L A T E  W E I G H T E D  A V E R A G E 1
+
+  This function calculates the weighted average of the available
+  available weight from the first entry until the one pointed
+  to by the end parameter passed to it.   If a weight was
+  entered more than 100 days ago, it is ignored (to reduce the
+  coumputation load for large data sets).
+ */
 float calculateWeightedAverage1(logEntry *end)
 {
   double startTime;
   float ave, weightSum, weight;
-  float p = 0.9;
+  float p = 0.9; /* The weight of each entry is reduced by p per day */
   logEntry *ptr;
 
   ptr = end;
@@ -363,9 +414,15 @@ float calculateWeightedAverage1(logEntry *end)
   }
   ave /= weightSum;
   return(ave);
-}
+} /* End of  C A L C U L A T E  W E I G H T E D  A V E R A G E 1 */
 
-float calculateWeightedAverage2(logEntry *end)
+/*
+  H A C K E R  D I E T  A V E R A G E
+
+  This routine calculates the weighted average using the "Hacker Diet"
+  algorithm.
+ */
+float hackerDietAverage(logEntry *end)
 {
   float ave;
   float p = 0.9;
@@ -379,17 +436,25 @@ float calculateWeightedAverage2(logEntry *end)
     ptr = ptr->next;
   }
   return(ave);
-}
+} /* End of  H A C K E R  D I E T  A V E R A G E */
 
+/*
+  C A L C U L A T E  W E I G H T E D  A V E R A G E
+
+  Select which function to use when calculating the weighted
+  average of the weight data values.
+ */
 float calculateWeightedAverage(logEntry *end)
 {
   if (hackerDietMode)
-    return(calculateWeightedAverage2(end));
+    return(hackerDietAverage(end));
   else
     return(calculateWeightedAverage1(end));
-}
+} /* End of  C A L C U L A T E  W E I G H T E D  A V E R A G E */
 
 /*
+  G E T  L I N E
+
   Read a line of text from the config file, and strip comments (flagged by #).
 */
 int getLine(int fD, int stripComments, char *buffer, int *eOF)
@@ -420,21 +485,17 @@ int getLine(int fD, int stripComments, char *buffer, int *eOF)
     return(TRUE);
   } else
     return(FALSE);
-}
-
-int tokenCheck(char *line, char *token, int type, void *value)
-
-#define INT_TOKEN     0
-#define DOUBLE_TOKEN  1
-#define FLOAT_TOKEN   2
-#define STRING_TOKEN  3
+} /* End of  G E T  L I N E */
 
 /*
+  T O K E N  C H E C K
+
   Scan "line" for "token".   If found, read the value into
   "value" as an integer, double or float, depending on "type".
 
   Return TRUE IFF the token is seen.
 */
+int tokenCheck(char *line, char *token, int type, void *value)
 {
   if (strstr(line, token)) {
     int nRead;
@@ -474,8 +535,13 @@ int tokenCheck(char *line, char *token, int type, void *value)
     return(TRUE);
   } else
     return(FALSE);
-}
+} /* End of  T O K E N  C H E C K */
 
+/*
+  R E A D  S E T T I N G S
+
+  Read the settings file and set the variables stored in it.
+*/
 void readSettings(char *fileName)
 {
   int eOF = FALSE;
@@ -485,6 +551,7 @@ void readSettings(char *fileName)
 
   settingsFD = open(fileName, O_RDONLY);
   if (settingsFD < 0) {
+    /* Not having a settings file is nonfatal - the default values will do. */
     perror("settings");
     return;
   }
@@ -506,16 +573,23 @@ void readSettings(char *fileName)
   }
   if (nonjudgementalColors)
     goodColor = badColor = OR_BLUE;
-}
+  close(settingsFD);
+} /* End of  R E A D  S E T T I N G S */
 
-#define BORDER_FACTOR_X (0.005)
-#define BORDER_FACTOR_Y (0.015)
+/*
+  C A L C  M A X I M A
 
+  Calculate the maximum and minimum weight and time (JD) values for the
+  entire data set from startTJD though all later entries.  The plot
+  maxima and minima are also computed.
+ */
 void calcMaxima(double startTJD)
 {
   if (nDataPoints == 1) {
     double aveJD, aveWeight;
-    
+
+    maxJD     = minJD     = logRoot->time;
+    maxWeight = minWeight = logRoot->weight;
     aveJD = (maxJD+minJD)*0.5;
     maxJD = aveJD + 5.0;
     minJD = aveJD - 5.0;
@@ -545,11 +619,11 @@ void calcMaxima(double startTJD)
       ptr = ptr->next;
     }
   }
-  if (maxJD == minJD) {
+  if ((maxJD - minJD) < 1.0) {
     maxJD += 1.0;
     minJD -= 1.0;
   }
-  if (maxWeight == minWeight) {
+  if ((maxWeight - minWeight) < 1.0) {
     maxWeight += 1.0;
     minWeight -= 1.0;
   }
@@ -563,8 +637,13 @@ void calcMaxima(double startTJD)
   }
   plotMaxY = maxWeight + BORDER_FACTOR_Y*(maxWeight-minWeight);
   plotMinY = minWeight - BORDER_FACTOR_Y*(maxWeight-minWeight);
-}
+  dprintf("maxWeight = %f minWeight = %f plotMaxY = %f plotMinY = %f\n",
+	 maxWeight, minWeight, plotMaxY, plotMinY);
+} /* End of  C A L C  M A X I M A */
+
 /*
+  R E A D  D A T A
+
   Read in the date, time and weight data from the data file.
 */
 void readData(char *fileName)
@@ -610,26 +689,12 @@ void readData(char *fileName)
     calcMaxima(0.0);
     close(dataFD);
   } else
-    perror("Data file could not be openned");
-  /*
-  if (debugMessagesOn) {
-    logEntry *ptr;
-
-    ptr = logRoot;
-    while (ptr != NULL) {
-      float est1, est2;
-
-      est1 = calculateWeightedAverage1(ptr);
-      est2 = calculateWeightedAverage2(ptr);
-      dprintf("%f   %f   %f   %f\n", 
-	     ptr->time - logRoot->time, ptr->weight, est1, est2);
-      ptr = ptr->next;
-    }
-  }
-  */
-}
+    perror("Data file could not be opened");
+} /* End of  R E A D  D A T A */
 
 /*
+  T J D  T O  D A T E
+
   Convert the Julian day into Calendar Date as per
   "Astronomical Algorithms" (Meeus)
 */
@@ -658,19 +723,27 @@ void tJDToDate(double tJD, int *year, int *month, int *day)
     *year = C - 4716;
   else
     *year = C - 4715;
-}
+} /* End of  T J D  T O  D A T E */
 
-#define LEFT_BORDER   (33)
-#define RIGHT_BORDER  (40)
-#define TOP_BORDER    (21)
-#define BOTTOM_BORDER (48)
+/*
+  S C R E E N  C O O R D I N A T E S
 
+  This function is called with plot coordinates in physical units
+  (time and weight) and it returns x, y coordinates in pixels on
+  the weight vs time plot.
+ */
 void screenCoordinates(double x, double y, int *ix, int *iy)
 {
   *ix = (x - plotMinX)*xScale + LEFT_BORDER;
   *iy = displayHeight - ((y - plotMinY)*yScale + BOTTOM_BORDER) - 2;
-}
+} /* End of  S C R E E N  C O O R D I N A T E S */
 
+/*
+  B M I
+
+  This function is passed a weight, and returns a Body Mass Index
+  value.
+ */
 double bMI(double weight)
 {
   double height, tWeight;
@@ -684,8 +757,14 @@ double bMI(double weight)
   else
     height = myHeight;
   return (tWeight*kg_per_lb/(height*height*m_per_in*m_per_in));
-}
+} /* End of  B M I */
 
+/*
+  W E I G H T
+
+  This function is called with a BMI value, and it returns the
+  corresponding weight.
+ */
 double weight(double bMI)
 {
   double height;
@@ -698,8 +777,15 @@ double weight(double bMI)
     return (bMI*height*height*m_per_in*m_per_in);
   else
     return (bMI*height*height*m_per_in*m_per_in/kg_per_lb);
-}
+} /* End of  W E I G H T */
 
+/*
+  C A L C  S T A T S
+
+  This function is passed a number of days, and it calculates the
+  weight loss statistics from a period that far in the past, until
+  the current time.
+ */
 int calcStats(int days, 
 	      float *startWeight, float *endWeight,
 	      float *maxWeight, float *minWeight, float *aveWeight)
@@ -765,8 +851,13 @@ int calcStats(int days,
   if (doAve)
     dprintf("Returning aveWeight = %f\n", *aveWeight);
   return(TRUE);
-}
+} /* End of  C A L C  S T A T S */
 
+/*
+  R E D R A W  S C R E E N
+
+  Redraw the weight vs time plot.
+ */
 void redrawScreen(void)
 {
   int nPoints = 0;
@@ -864,6 +955,16 @@ void redrawScreen(void)
 		    &tWidth, &tHeight, pixmap, displayWidth/2, 10, 0.0, TRUE, 0, OR_BLACK);
     renderPangoText(scratchString, OR_WHITE, SMALL_PANGO_FONT,
 		    &tWidth, &tHeight, pixmap, displayWidth/2, 10, 0.0, TRUE, 0, OR_BLACK);
+    if (showTarget) {
+      int x0, y0, x1, y1;
+      
+      /* Draw a thick green line at the target weight */
+      screenCoordinates(plotMinX, myTarget, &x0, &y0);
+      screenCoordinates(plotMaxX, myTarget, &x1, &y1);
+      gdk_draw_line(pixmap, gC[OR_GREEN], x0, y0-1, x1, y1-1);
+      gdk_draw_line(pixmap, gC[OR_GREEN], x0, y0,   x1, y1);
+      gdk_draw_line(pixmap, gC[OR_GREEN], x0, y0+1, x1, y1+1);
+    }
 
     /* Y axis labels */
 
@@ -871,7 +972,7 @@ void redrawScreen(void)
     xScale = ((double)displayWidth - LEFT_BORDER - RIGHT_BORDER - 1)/(plotMaxX - plotMinX);
     yScale = ((double)displayHeight - TOP_BORDER - BOTTOM_BORDER - 4)/(plotMaxY - plotMinY);
     weightStart = (int)(minWeight + 0.5);
-    bMIStart    = (int)(10.0*(int)(bMI(minWeight) + 0.5));
+    bMIStart    = (int)(10.0*(int)(bMI(minWeight) - 0.5));
     weightEnd   = (int)(maxWeight + 0.5);
     bMIEnd      = (int)(10.0*(int)(bMI(maxWeight) + 0.5));
     if (weightStart >= weightEnd) {
@@ -954,15 +1055,6 @@ void redrawScreen(void)
 	  gdk_draw_line(pixmap, gC[OR_WHITE], x, y, x-stub, y);
 	}
       }
-      if (showTarget) {
-	int x0, y0, x1, y1;
-
-	screenCoordinates(plotMinX, myTarget, &x0, &y0);
-	screenCoordinates(plotMaxX, myTarget, &x1, &y1);
-	gdk_draw_line(pixmap, gC[OR_GREEN], x0, y0-1, x1, y1-1);
-	gdk_draw_line(pixmap, gC[OR_GREEN], x0, y0,   x1, y1);
-	gdk_draw_line(pixmap, gC[OR_GREEN], x0, y0+1, x1, y1+1);
-      }
     }
     /* X axis labelling */
     tJDToDate(logRoot->time, &sYear, &sMonth, &sDay);
@@ -1118,11 +1210,15 @@ void redrawScreen(void)
 		    pixmap,
 		    0,0,0,0,
 		    displayWidth, displayHeight);
-}
+} /* End of  R E D R A W  S C R E E N */
 
 /*
+  E X P O S E  E V E N T
+
   This function is called when a portion of the drawing area
-  is exposed.
+  is exposed.  The plot is put together through calls which
+  write on pixmap, which is copied to the drawing area for
+  display.
 */
 static gboolean exposeEvent(GtkWidget *widget, GdkEventExpose *event)
 {
@@ -1134,9 +1230,11 @@ static gboolean exposeEvent(GtkWidget *widget, GdkEventExpose *event)
                      event->area.x, event->area.y,
                      event->area.width, event->area.height);
   return(FALSE);
-}
+} /* End of  E X P O S E  E V E N T */
 
 /*
+  C O N F I G U R E  E V E N T
+
   This function creates and initializes a backing pixmap
   of the appropriate size for the drawing area.
 */
@@ -1155,9 +1253,11 @@ static int configureEvent(GtkWidget *widget, GdkEventConfigure *event)
 	  displayWidth, displayHeight);
   redrawScreen();
   return TRUE;
-}
+} /* End of  C O N F I G U R E  E V E N T */
 
 /*
+  M A K E  G R A P H I C  C O N T E X T S
+
   Make all the Graphic Contexts the program will need.
 */
 static void makeGraphicContexts(GtkWidget *widget)
@@ -1191,32 +1291,11 @@ static void makeGraphicContexts(GtkWidget *widget)
       exit(ERROR_EXIT);
     }
   }
-}
+} /* End of  M A K E  G R A P H I C  C O N T E X T S */
 
 /*
-  Set up the fonts I need.
-*/
-static void makeFonts(GtkWidget *widget)
-{
-  smallFont =
-    gdk_font_load("-misc-fixed-medium-r-semicondensed--13-100-100-100-c-60-iso8859-1");
-  if (smallFont == NULL) {
-    fprintf(stderr, "Error making smallFont\n");
-    exit(ERROR_EXIT);
-  }
-  bigFont =
-    gdk_font_load("-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1");
-  if (bigFont == NULL) {
-    fprintf(stderr, "Error making bigFont\n");
-    exit(ERROR_EXIT);
-  }
-}
+  C R E A T E  T O U C H  S E L E C T O R
 
-GtkWidget *selector;
-int minSelectorWeight = 50;
-int maxSelectorWeight = 300;
-
-/*
   This function creates the selector for the weight "picker".
 */
 static GtkWidget *createTouchSelector(void)
@@ -1263,8 +1342,14 @@ static GtkWidget *createTouchSelector(void)
                                                      GTK_TREE_MODEL(model), TRUE);
   g_object_set(G_OBJECT(column), "text-column", 0, NULL);  
   return selector;
-}
+} /* End of  C R E A T E  T O U C H  S E L E C T O R */
 
+/*
+  W R I T E  F I L E
+
+  Write the data entries to the file.   The function is called
+  twice, to write the main file and its backup.
+ */
 void writeFile(FILE *dataFile)
 {
  int day, month, year, hours, minutes;
@@ -1287,8 +1372,13 @@ void writeFile(FILE *dataFile)
     ptr = ptr->next;
   }
   fclose(dataFile);
-}
+} /* End of  W R I T E  F I L E */
 
+/*
+  W R I T E  D A T A  F I L E
+
+  Write the main data file and its backup.
+ */
 void writeDataFile(void)
 {
   FILE *dataFile;
@@ -1303,8 +1393,13 @@ void writeDataFile(void)
     fprintf(stderr, "Cannot write backup data file (%s)\n", backupFile);
   else
     writeFile(dataFile);
-}
+} /* End of  W R I T E  D A T A  F I L E */
 
+/*
+  I N S E R T  Q U E U E  E N T R Y
+
+  Insert a new value into the data queue
+ */
 void insertQueueEntry(double fJD, float weight, char *comment)
 {
   logEntry  *newEntry;
@@ -1338,18 +1433,27 @@ void insertQueueEntry(double fJD, float weight, char *comment)
   dprintf("Post: minJD = %f, maxJD = %f, minWeight = %f maxWeight = %f\n",
 	 minJD, maxJD, minWeight, maxWeight);
   writeDataFile();
-}
+} /* End of  I N S E R T  Q U E U E  E N T R Y */
 
-GtkWidget *dataEntryStackable, *settingsStackable, *logStackable;
-int cancelled;
+/*
+  D A T A  E N T R Y  C A N C E L  C A L L B A C K
 
+  This fuction is called if the cancel putton is pushed on the
+  Data Entry page.   That is the only way to exit from that page
+  without entering new data.
+ */
 void dataEntryCancelCallback(GtkButton *button, gpointer userData)
 {
   cancelled = TRUE;
   gtk_widget_destroy(dataEntryStackable);
-}
+} /* End of  D A T A  E N T R Y  C A N C E L  C A L L B A C K */
 
-GtkWidget *dateButton, *timeButton;
+/*
+  C H E C K  D A T A
+
+  This function reads the values from the widgets on the Data Entry
+  page, to put together the values for a new entry in the weight log.
+ */
 void checkData(void)
 {
   int iJD, i;
@@ -1383,8 +1487,15 @@ void checkData(void)
     insertQueueEntry(fJD, weight, comment);
   }
   redrawScreen();
-}
+} /* End of  C H E C K  D A T A */
 
+/*
+  D A T A  E N T R Y  B U T T O N  C L I C K E D
+
+  This funtion is called if Data Entry is selected fromt the main menu.
+  It builds and displays the widgets on the Data Entry page, and
+  loads them with the default values.
+ */
 void dataEntryButtonClicked(GtkButton *button, gpointer userData)
 {
   static GtkWidget *weightSelector, *dataEntryTable;
@@ -1438,30 +1549,23 @@ void dataEntryButtonClicked(GtkButton *button, gpointer userData)
   
   gtk_container_add(GTK_CONTAINER(dataEntryStackable), dataEntryTable);
   gtk_widget_show_all(dataEntryStackable);
-}
+} /* End of  D A T A  E N T R Y  B U T T O N  C L I C K E D */
 
-void chartButtonClicked(GtkButton *button, gpointer userData)
-{
-  dprintf("in chartButtonClicked()\n");
-}
-
-GtkWidget *trendStackable;
-int mainBoxInTrendStackable = FALSE;
-
-typedef struct logCell {
-  GdkPoint        box[4];
-  logEntry       *ptr;
-  struct logCell *next;
-} logCell;
-logCell *logCellRoot = NULL;
-logCell *lastLogCell = NULL;
-int logDisplayed = FALSE;
+/*
+  T R E N D  S T A C K A B L E  D E S T R O Y E D
 
+  This function is called when the Trends or Log page is exited.   It causes
+the program to re-display the main plot page.
+ */
 void trendStackableDestroyed(void)
 {
+  /*
+    If the Log page is the one which was displayed, we must do the garbage
+    collection for the dynamically allocated structures.
+   */
   if (logDisplayed) {
     logCell *ptr, *tptr;
-
+    
     ptr = logCellRoot;
     while (ptr != NULL) {
       tptr = ptr;
@@ -1482,12 +1586,11 @@ void trendStackableDestroyed(void)
 		    pixmap,
 		    0,0,0,0,
 		    displayWidth, displayHeight);
-}
-
-unsigned char normalYear[12] = {0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5};
-unsigned char leapYear[2] = {6, 2};
+} /* End of  T R E N D  S T A C K A B L E  D E S T R O Y E D */
 
 /*
+  D A Y  O F  W E E K
+
   Return the day of the week (0->6) for year, month, day
 */
 int dayOfWeek(int year, int month, int day)
@@ -1512,8 +1615,14 @@ int dayOfWeek(int year, int month, int day)
     m = (int)normalYear[month-1];
   weekDay = (c + y + m + day) % 7;
   return(weekDay);
-}
+} /* End of  D A Y  O F  W E E K */
 
+/*
+  L O G  C A L L B A C K
+
+  This routine builds the page which displays the weight values for a particular
+  month.
+ */
 void logCallback(void)
 {
   int year, month, tWidth, tHeight;
@@ -1624,8 +1733,16 @@ void logCallback(void)
     }
     ptr = ptr->next;
   }
-}
+  renderPangoText("Click on an entry, if you wish to edit or delete it.", OR_BLUE, MEDIUM_PANGO_FONT,
+		  &tWidth, &tHeight, pixmap, displayWidth/2, displayHeight-20, 0.0, TRUE, 0, OR_BLACK);
+} /* End of  L O G  C A L L B A C K */
 
+/*
+  L O G  B U T T O N  C L I C K E D
+
+  This function builds and displays a selector which shows every month
+  for which data has been entered.
+ */
 void logButtonClicked(GtkButton *button, gpointer userData)
 {
   int lastMonth = -1;
@@ -1674,8 +1791,14 @@ void logButtonClicked(GtkButton *button, gpointer userData)
     gtk_container_add(GTK_CONTAINER(logStackable), logTable);
     gtk_widget_show_all(logStackable);
   }
-}
+} /* End of  L O G  B U T T O N  C L I C K E D */
 
+/*
+  T R E N D  B U T T O N  C L I C K E D
+
+  This function builds an displays the page which shows weight trends for
+  the last week, fortnight, month etc.
+ */
 void trendButtonClicked(GtkButton *button, gpointer userData)
 {
   int tWidth, tHeight, day, month, year, line;
@@ -2004,13 +2127,13 @@ void trendButtonClicked(GtkButton *button, gpointer userData)
 	break;
       }
   }
-}
+} /* End of  T R E N D  B U T T O N  C L I C K E D */
 
-void utilitiesButtonClicked(GtkButton *button, gpointer userData)
-{
-  dprintf("in utilitiesButtonClicked()\n");
-}
+/*
+  W R I T E  S E T T I N G S
 
+  Write the user-selectable parameters into the settings file.
+ */
 void writeSettings(void)
 {
   FILE *settings;
@@ -2031,20 +2154,20 @@ void writeSettings(void)
   fprintf(settings, "SHOW_TARGET %d\n",           showTarget);
   fprintf(settings, "PLOT_INTERVAL %d\n",         plotInterval);
   fclose(settings);
-}
+} /* End of  W R I T E  S E T T I N G S */
 
-GtkWidget *heightSpinLabel, *heightSpin, *weightUnitLabel, *heightUnitLabel,
-  *dateFormatLabel, *poundsButton, *kgButton, *inButton, *cmButton, *dayButton,
-  *monthButton, *nonjudgementalButton, *hackerDietButton, *showCommentsButton,
-  *plotWeekButton, *plotMonthButton, *plotQuarterButton, *plot6MonthButton,
-  *plotYearButton, *plotHistoryButton, *targetSpinLabel, *targetSpin, *showTargetButton;
+/*
+  C H E C K  S E T T I N G S
 
+  Read the widgets from the Settings page, and set the user-selectable
+  variables.
+*/
 void checkSettings(void)
 {
   dprintf("In checkSettings\n");
   myHeight = gtk_spin_button_get_value((GtkSpinButton *)heightSpin);
   myTarget = gtk_spin_button_get_value((GtkSpinButton *)targetSpin);
-  if (GTK_TOGGLE_BUTTON(plotWeekButton)->active)
+  if (GTK_TOGGLE_BUTTON(plotFortnightButton)->active)
     plotInterval = 14;
   else if (GTK_TOGGLE_BUTTON(plotMonthButton)->active)
     plotInterval = 30;
@@ -2090,8 +2213,13 @@ void checkSettings(void)
     monthFirst = FALSE;
   writeSettings();
   redrawScreen();
-}
+} /* End of  C H E C K  S E T T I N G S */
 
+/*
+  S E T T I N G S  B U T T O N  C L I C K E D
+
+  This function builds and displays the widgets for the Settings page.
+ */
 void settingsButtonClicked(GtkButton *button, gpointer userData)
 {
   static GtkWidget *settingsTable;
@@ -2196,8 +2324,8 @@ void settingsButtonClicked(GtkButton *button, gpointer userData)
   gtk_table_attach(GTK_TABLE(settingsTable), showCommentsButton, 2, 3, 5, 6,
 		   GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
-  plotWeekButton = gtk_radio_button_new_with_label(NULL, "Plot Last 2 Weeks");
-  plotGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(plotWeekButton));
+  plotFortnightButton = gtk_radio_button_new_with_label(NULL, "Plot Last Fortnight");
+  plotGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(plotFortnightButton));
   plotMonthButton = gtk_radio_button_new_with_label(plotGroup, "Plot Last Month");
   plotGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(plotMonthButton));
   plotQuarterButton = gtk_radio_button_new_with_label(plotGroup, "Plot Last Quarter");
@@ -2209,8 +2337,8 @@ void settingsButtonClicked(GtkButton *button, gpointer userData)
   plotHistoryButton = gtk_radio_button_new_with_label(plotGroup, "Plot Entire History");
   plotGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(plotHistoryButton));
   switch (plotInterval) {
-  case 7:
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plotWeekButton), TRUE);
+  case 14:
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plotFortnightButton), TRUE);
     break;
   case 30:
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plotMonthButton), TRUE);
@@ -2227,7 +2355,7 @@ void settingsButtonClicked(GtkButton *button, gpointer userData)
   default:
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plotHistoryButton), TRUE);
   }
-  gtk_table_attach(GTK_TABLE(settingsTable), plotWeekButton, 0, 1, 6, 7,
+  gtk_table_attach(GTK_TABLE(settingsTable), plotFortnightButton, 0, 1, 6, 7,
 		   GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_table_attach(GTK_TABLE(settingsTable), plotMonthButton, 1, 2, 6, 7,
 		   GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
@@ -2245,14 +2373,26 @@ void settingsButtonClicked(GtkButton *button, gpointer userData)
 		   G_CALLBACK(checkSettings), NULL);
   gtk_container_add(GTK_CONTAINER(settingsStackable), settingsTable);
   gtk_widget_show_all(settingsStackable);
-}
+} /* End of  S E T T I N G S  B U T T O N  C L I C K E D */
 
+/*
+  H E L P  B U T T O N  C L I C K E D
+
+  This fucntion is called if Help is chosen fromt he main menu.
+ */
 void helpButtonClicked(GtkButton *button, gpointer userData)
 {
   dprintf("in helpButtonClicked()\n");
+  /* Send a dbus command to open the maeFat wiki page in the default browser */
   system("/usr/bin/dbus-send --system --type=method_call  --dest=\"com.nokia.osso_browser\" /com/nokia/osso_browser/request com.nokia.osso_browser.load_url string:\"wiki.maemo.org/maeFat\"");
-}
+} /* End of  H E L P  B U T T O N  C L I C K E D */
 
+/*
+  A B O U T  B U T T O N  C L I C K E D
+
+  This function is called if About is selected from the main menu.
+  It just prints a brief message about the program.
+ */
 void aboutButtonClicked(GtkButton *button, gpointer userData)
 {
   static int firstCall = TRUE;
@@ -2262,7 +2402,7 @@ void aboutButtonClicked(GtkButton *button, gpointer userData)
   dprintf("in aboutButtonClicked()\n");
   if (firstCall) {
     aboutBuffer = gtk_text_buffer_new(NULL);
-    gtk_text_buffer_set_text(aboutBuffer, "maeFat Version 1.1\nCopyright (C) 2011, Ken Young\n\nThis program helps you keep track of your weight.\nThis is free, open source software, released under GPL version 2.\n\nPlease send comments, questions and feature requests to\norrery.moko@gmail.com", -1);
+    gtk_text_buffer_set_text(aboutBuffer, "maeFat Version 1.2\nCopyright (C) 2011, Ken Young\n\nThis program helps you keep track of your weight.\nThis is free, open source software, released under GPL version 2.\n\nPlease send comments, questions and feature requests to\norrery.moko@gmail.com", -1);
     aboutTextWidget = hildon_text_view_new(); 
     ((GtkTextView *)aboutTextWidget)->editable =
       ((GtkTextView *)aboutTextWidget)->cursor_visible = FALSE;
@@ -2273,9 +2413,13 @@ void aboutButtonClicked(GtkButton *button, gpointer userData)
   aboutStackable = hildon_stackable_window_new();
   gtk_container_add(GTK_CONTAINER(aboutStackable), aboutTextWidget);
   gtk_widget_show_all(aboutStackable);
-}
+} /* End of  A B O U T  B U T T O N  C L I C K E D */
 
-HildonAppMenu *hildonMenu;
+/*
+  M A K E  H I L D O N  B U T T O N S
+
+  This function builds the main menu for the app.
+ */
 static void makeHildonButtons(void)
 {
   HildonSizeType buttonSize = HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH;
@@ -2288,14 +2432,6 @@ static void makeHildonButtons(void)
   g_signal_connect(G_OBJECT(dataEntryButton), "clicked", G_CALLBACK(dataEntryButtonClicked), NULL);
   hildon_app_menu_append(hildonMenu, GTK_BUTTON(dataEntryButton));
   
-  /* Chart Button */
-  /*
-  chartButton = hildon_gtk_button_new(buttonSize);
-  gtk_button_set_label(GTK_BUTTON(chartButton), "Chart");
-  g_signal_connect(G_OBJECT(chartButton), "clicked", G_CALLBACK(chartButtonClicked), NULL);
-  hildon_app_menu_append(hildonMenu, GTK_BUTTON(chartButton));
-  */
-
   /* Log  Button */
   logButton = hildon_gtk_button_new(buttonSize);
   gtk_button_set_label(GTK_BUTTON(logButton), "Log");
@@ -2308,14 +2444,6 @@ static void makeHildonButtons(void)
   g_signal_connect(G_OBJECT(trendButton), "clicked", G_CALLBACK(trendButtonClicked), NULL);
   hildon_app_menu_append(hildonMenu, GTK_BUTTON(trendButton));
 
-  /* Utilities Button */
-  /*
-  utilitiesButton = hildon_gtk_button_new(buttonSize);
-  gtk_button_set_label(GTK_BUTTON(utilitiesButton), "Utilities");
-  g_signal_connect(G_OBJECT(utilitiesButton), "clicked", G_CALLBACK(utilitiesButtonClicked), NULL);
-  hildon_app_menu_append(hildonMenu, GTK_BUTTON(utilitiesButton));
-  */
-
   /* Settings Button */
   settingsButton = hildon_gtk_button_new(buttonSize);
   gtk_button_set_label(GTK_BUTTON(settingsButton), "Settings");
@@ -2336,29 +2464,42 @@ static void makeHildonButtons(void)
 
   hildon_stackable_window_set_main_menu((HildonStackableWindow *)window, hildonMenu);
   gtk_widget_show_all(GTK_WIDGET(hildonMenu));
-}
+} /* End of  M A K E  H I L D O N  B U T T O N S */
 
+/*
+  B U T T O N  P R E S S  E V E N T
+
+  This function is called when the screen is pressed
+ */
 static gboolean buttonPressEvent(GtkWidget *widget, GdkEventButton *event)
 {
   dprintf("In buttonPressEvent\n");
   return(TRUE);
-}
+} /* End of  B U T T O N  P R E S S  E V E N T */
 
-int deleted;
-logEntry *editPtr;
-GtkWidget *dateEditButton, *timeEditButton, *dataEditStackable;
+/*
+  D A T A  E D I T  D E L E T E  C A L L B A C K
 
+  This function is called if the delete button is pressed on the log
+  edit page.
+ */
 void dataEditDeleteCallback(GtkButton *button, gpointer userData)
 {
   deleted = TRUE;
   gtk_widget_destroy(dataEditStackable);
-}
+} /* End of  D A T A  E D I T  D E L E T E  C A L L B A C K */
 
+/*
+  E D I T  D A T A
+
+  This function is called when a log entry has been edited.  It gets
+  the new values from the widgets on the edit page.
+ */
 void editData(void)
 {
   if (deleted) {
     if (editPtr->last == NULL)
-      logRoot = editPtr;
+      logRoot = editPtr->next;
     else {
       (editPtr->last)->next = editPtr->next;
       (editPtr->next)->last = editPtr->last;
@@ -2400,8 +2541,14 @@ void editData(void)
   }
   writeDataFile();
   gtk_widget_destroy(trendStackable);
-}
+} /* End of  E D I T  D A T A */
 
+/*
+  C R E A T E  E D I T  P A G E
+
+  This function creates and displays the widgets needed for the Edit
+  page.
+ */
 void createEditPage(logEntry *ptr)
 {
   double dayFrac;
@@ -2455,8 +2602,17 @@ void createEditPage(logEntry *ptr)
 		   GTK_EXPAND, GTK_EXPAND, 0, 0);
   gtk_container_add(GTK_CONTAINER(dataEditStackable), dataEditTable);
   gtk_widget_show_all(dataEditStackable);
-}
+} /* End of  C R E A T E  E D I T  P A G E */
 
+/*
+  B U T T O N  R E L E A S E  E V E N T
+
+  This function is called when a button press in the screen is released.
+  It does nothing unless the Log page is displayed.   If the Log page
+  is displayed, it checked to see if the press event corredinates
+  corespond to one of the weight entry buttons.   If so, it builds an
+  edit page, otherwise nothing happens.
+ */
 static gboolean buttonReleaseEvent(GtkWidget *widget, GdkEventButton *event)
 {
   dprintf("In buttonReleaseEvent\n");
@@ -2467,6 +2623,7 @@ static gboolean buttonReleaseEvent(GtkWidget *widget, GdkEventButton *event)
 
     x = event->x; y = event->y;
     dprintf("The log is displayed (%d, %d)\n", x, y);
+    /* Loop through all the data cells displayed on the Log page */
     ptr = logCellRoot;
     while ((ptr != NULL) && (!found)) {
       if ((ptr->box[0].x <= x) && (ptr->box[0].y <= y) && (ptr->box[2].x >= x) && (ptr->box[2].y >= y))
@@ -2483,7 +2640,7 @@ static gboolean buttonReleaseEvent(GtkWidget *widget, GdkEventButton *event)
     }
   }
   return(TRUE);
-}
+} /* End of  B U T T O N  R E L E A S E  E V E N T */
 
 int main(int argc, char **argv)
 {
@@ -2560,7 +2717,6 @@ int main(int argc, char **argv)
 		   G_CALLBACK(buttonPressEvent), NULL);
   gtk_widget_show(window);
   makeGraphicContexts(window);
-  makeFonts(window);
   gtk_widget_set_events(drawingArea,
 			GDK_EXPOSURE_MASK       | GDK_BUTTON_PRESS_MASK  |
 			GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK);
